@@ -97,6 +97,7 @@ typedef struct r_egg_t {
 	RList *list;
 	//RList *shellcodes; // XXX is plugins nao?
 	RAsm *rasm;
+	RAnal *anal;/// XXX this is temporary hack until asm can use arch
 	RSyscall *syscall;
 	REggLang lang;
 	Sdb *db;
@@ -139,7 +140,7 @@ EOF
 #define R_EGG_OS_DEFAULT R_EGG_OS_OSX
 #define R_EGG_OS_NAME "darwin"
 #define R_EGG_FORMAT_DEFAULT "mach0"
-#elif __WINDOWS__
+#elif R2__WINDOWS__
 #define R_EGG_OS_DEFAULT R_EGG_OS_W32
 #define R_EGG_OS_NAME "windows"
 #define R_EGG_FORMAT_DEFAULT "pe"
@@ -181,29 +182,30 @@ typedef struct r_egg_emit_t {
 } REggEmit;
 
 #ifdef R_API
-R_API REgg *r_egg_new (void);
+R_API REgg *r_egg_new(void);
 R_API void r_egg_lang_init(REgg *egg);
 R_API void r_egg_lang_free(REgg *egg);
-R_API char *r_egg_to_string (REgg *egg);
-R_API void r_egg_free (REgg *egg);
-R_API int r_egg_add (REgg *a, REggPlugin *foo);
-R_API void r_egg_reset (REgg *egg);
-R_API int r_egg_setup(REgg *egg, const char *arch, int bits, int endian, const char *os);
-R_API int r_egg_include(REgg *egg, const char *file, int format);
+R_API char *r_egg_tostring(REgg *egg);
+R_API void r_egg_free(REgg *egg);
+R_API bool r_egg_add(REgg *a, REggPlugin *foo);
+R_API void r_egg_reset(REgg *egg);
+R_API bool r_egg_setup(REgg *egg, const char *arch, int bits, int endian, const char *os);
+R_API bool r_egg_include(REgg *egg, const char *file, int format);
+R_API bool r_egg_include_str(REgg *egg, const char *arg);
 R_API void r_egg_load(REgg *egg, const char *code, int format);
 R_API void r_egg_syscall(REgg *egg, const char *arg, ...) R_PRINTF_CHECK(2, 3);
 R_API void r_egg_alloc(REgg *egg, int n);
 R_API void r_egg_label(REgg *egg, const char *name);
-R_API int r_egg_raw(REgg *egg, const ut8 *b, int len);
-R_API int r_egg_encode(REgg *egg, const char *name);
-R_API int r_egg_shellcode(REgg *egg, const char *name);
+R_API bool r_egg_raw(REgg *egg, const ut8 *b, int len);
+R_API bool r_egg_encode(REgg *egg, const char *name);
+R_API bool r_egg_shellcode(REgg *egg, const char *name);
 #define r_egg_get_shellcodes(x) x->plugins
-R_API void r_egg_option_set (REgg *egg, const char *k, const char *v);
-R_API char *r_egg_option_get (REgg *egg, const char *k);
+R_API void r_egg_option_set(REgg *egg, const char *k, const char *v);
+R_API char *r_egg_option_get(REgg *egg, const char *k);
 R_API void r_egg_if(REgg *egg, const char *reg, char cmp, int v);
 R_API void r_egg_printf(REgg *egg, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
-R_API int r_egg_compile(REgg *egg);
-R_API int r_egg_padding (REgg *egg, const char *pad);
+R_API bool r_egg_compile(REgg *egg);
+R_API bool r_egg_padding(REgg *egg, const char *pad);
 R_API bool r_egg_assemble(REgg *egg);
 R_API bool r_egg_assemble_asm(REgg *egg, char **asm_list);
 R_API void r_egg_pattern(REgg *egg, int size);
@@ -215,17 +217,17 @@ R_API char *r_egg_get_assembly(REgg *egg);
 R_API void r_egg_append(REgg *egg, const char *src);
 R_API int r_egg_run(REgg *egg);
 R_API int r_egg_run_rop(REgg *egg);
-R_API int r_egg_patch(REgg *egg, int off, const ut8 *b, int l);
+R_API bool r_egg_patch(REgg *egg, int off, const ut8 *b, int l);
 R_API void r_egg_finalize(REgg *egg);
 
-/* r_egg_Cfile.c */
-R_API char* r_egg_Cfile_parser(const char *file, const char *arch, const char *os, int bits);
+/* r_egg_cfile.c */
+R_API char* r_egg_cfile_parser(const char *file, const char *arch, const char *os, int bits);
 
 /* lang.c */
 R_API char *r_egg_mkvar(REgg *egg, char *out, const char *_str, int delta);
 R_API int r_egg_lang_parsechar(REgg *egg, char c);
-R_API void r_egg_lang_include_path (REgg *egg, const char *path);
-R_API void r_egg_lang_include_init (REgg *egg);
+R_API void r_egg_lang_include_path(REgg *egg, const char *path);
+R_API void r_egg_lang_include_init(REgg *egg);
 
 /* plugin pointers */
 extern REggPlugin r_egg_plugin_xor;
