@@ -8,6 +8,7 @@ extern "C" {
 #include "r_version.h"
 
 #define R_CHECKS_LEVEL 2
+#define R_CRITICAL_ENABLED 0
 #define DEBUGGER 1
 #define HAVE_DECL_ADDR_NO_RANDOMIZE 1
 #define HAVE_ARC4RANDOM_UNIFORM 0
@@ -18,6 +19,8 @@ extern "C" {
 #define WANT_THREADS 0
 #define WANT_CAPSTONE 1
 
+#define WITH_STATIC_THEMES 0
+
 #define HAVE_GPERF 0
 #if (HAVE_GPERF) == 1
 #define HAVE_GPERF 0
@@ -31,19 +34,19 @@ extern "C" {
 #define R2_LIBDIR "/usr/lib"
 #define R2_INCDIR "/usr/include/libr"
 #define R2_DATDIR "/usr/share"
-#define R2_WWWROOT "/usr/share/radare2/5.7.8/www"
-#define R2_PLUGINS "lib/radare2/5.7.8"
-#define R2_EXTRAS "lib/radare2-extras/5.7.8"
-#define R2_BINDINGS "lib/radare2-bindings/5.7.8"
+#define R2_WWWROOT "/usr/share/radare2/5.8.2/www"
+#define R2_PLUGINS "lib/radare2/5.8.2"
+#define R2_EXTRAS "lib/radare2-extras/5.8.2"
+#define R2_BINDINGS "lib/radare2-bindings/5.8.2"
 #define R2_DATDIR_R2 "share/radare2"
-#define R2_SDB "share/radare2/5.7.8"
-#define R2_ZIGNS "share/radare2/5.7.8/zigns"
-#define R2_THEMES "share/radare2/5.7.8/cons"
+#define R2_SDB "share/radare2/5.8.2"
+#define R2_ZIGNS "share/radare2/5.8.2/zigns"
+#define R2_THEMES "share/radare2/5.8.2/cons"
 #define R2_FORTUNES R_JOIN_3_PATHS ("/usr/share", "doc", "radare2")
-#define R2_FLAGS "share/radare2/5.7.8/flag"
-#define R2_HUD "share/radare2/5.7.8/hud"
+#define R2_FLAGS "share/radare2/5.8.2/flag"
+#define R2_HUD "share/radare2/5.8.2/hud"
 #else
-#if __WINDOWS__ || _MSC_VER
+#if R2__WINDOWS__ || _MSC_VER
 #define R2_PREFIX "."
 #define R2_ETCDIR "etc"
 #define R2_LIBDIR "lib"
@@ -78,27 +81,6 @@ extern "C" {
 #define R2_SDB_MAGIC        R_JOIN_2_PATHS (R2_SDB, "magic")
 #define R2_SDB_FORMAT       R_JOIN_2_PATHS (R2_SDB, "format")
 
-#define R2_HOME_CONFIGDIR   R_JOIN_2_PATHS (".config", "radare2")
-#define R2_HOME_DATADIR     R_JOIN_3_PATHS (".local", "share", "radare2")
-#define R2_HOME_CACHEDIR    R_JOIN_2_PATHS (".cache", "radare2")
-
-#define R2_HOME_THEMES      R_JOIN_2_PATHS (R2_HOME_DATADIR, "cons")
-#define R2_HOME_FORTUNES    R_JOIN_2_PATHS (R2_HOME_DATADIR, "fortunes")
-#define R2_HOME_PLUGINS     R_JOIN_2_PATHS (R2_HOME_DATADIR, "plugins")
-#define R2_HOME_ZIGNS       R_JOIN_2_PATHS (R2_HOME_DATADIR, "zigns")
-#define R2_HOME_PDB         R_JOIN_2_PATHS (R2_HOME_DATADIR, "pdb")
-#define R2_HOME_PROJECTS    R_JOIN_2_PATHS (R2_HOME_DATADIR, "projects")
-#define R2_HOME_WWWROOT     R_JOIN_2_PATHS (R2_HOME_DATADIR, "www")
-#define R2_HOME_SDB_FCNSIGN R_JOIN_2_PATHS (R2_HOME_DATADIR, "fcnsign")
-#define R2_HOME_SDB_FORMAT  R_JOIN_2_PATHS (R2_HOME_DATADIR, "format")
-#define R2_HOME_HUD         R_JOIN_2_PATHS (R2_HOME_DATADIR, "hud")
-#define R2_HOME_BINRC       R_JOIN_2_PATHS (R2_HOME_DATADIR, "rc.d")
-#define R2_HOME_BIN         R_JOIN_3_PATHS (R2_HOME_DATADIR, "prefix", "bin")
-
-#define R2_HOME_HISTORY     R_JOIN_2_PATHS (R2_HOME_CACHEDIR, "history")
-
-#define R2_HOME_RC          R_JOIN_2_PATHS (R2_HOME_CONFIGDIR, "radare2rc")
-#define R2_HOME_RC_DIR      R_JOIN_2_PATHS (R2_HOME_CONFIGDIR, "radare2rc.d")
 #define R2_GLOBAL_RC        R_JOIN_2_PATHS (R2_DATDIR_R2, "radare2rc")
 
 #define HAVE_LIB_MAGIC 1
@@ -110,6 +92,10 @@ extern "C" {
 #define HAVE_LIB_SSL 0
 #endif
 
+#ifndef WANT_SSL_CRYPTO
+#define WANT_SSL_CRYPTO 0
+#endif
+
 #define HAVE_LIBUV 1
 
 #if __MINGW32__
